import androidx.compose.foundation.Image
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.aspectRatio
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material3.Button
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableFloatStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.nestedscroll.NestedScrollConnection
import androidx.compose.ui.input.nestedscroll.NestedScrollSource
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.unit.Velocity
import androidx.compose.ui.unit.dp
import androidx.navigation.NavHostController
import details.ingredients
import details.ingredientsHeader
import details.recipeHeader
import details.steps
import details.stepsHeader
import model.Recipe
import org.jetbrains.compose.resources.painterResource
import sensor.Listener
import sensor.SensorData
import sensor.SensorManager
import view.RecipeScaleSlider


@Composable
fun RecipeDetailsLarge(
    recipe: Recipe,
    goBack: () -> Unit,
    sensorManager: SensorManager?,
    navController: NavHostController,
) {
    val sensorDataLive = remember { mutableStateOf(SensorData(0.0f, 0.0f)) }

    sensorManager?.registerListener(object : Listener {
        override fun onUpdate(sensorData: SensorData) {
            sensorDataLive.value = sensorData
        }
    })

    val nestedScrollConnection = remember {
        object : NestedScrollConnection {
            override fun onPreScroll(
                available: Offset, source: NestedScrollSource
            ): Offset {
                return Offset.Zero
            }

            override fun onPostScroll(
                consumed: Offset, available: Offset, source: NestedScrollSource
            ): Offset {
                val delta = available.y
                return super.onPostScroll(consumed, available, source)
            }

            override suspend fun onPreFling(available: Velocity): Velocity {
                return super.onPreFling(available)
            }
        }
    }

    Box(
        modifier = Modifier.fillMaxSize()
            .background(if (recipe.bgColor == MaterialTheme.colorScheme.primaryContainer) MaterialTheme.colorScheme.secondaryContainer else MaterialTheme.colorScheme.background)
    ) {
        var reduce by remember { mutableStateOf(false) }
        var slider by remember { mutableFloatStateOf(1f) }

        Row {
            Box(
                modifier = Modifier.fillMaxSize()
                    .background(color = if (recipe.bgColor == MaterialTheme.colorScheme.primaryContainer) MaterialTheme.colorScheme.secondaryContainer else MaterialTheme.colorScheme.background)
                    .weight(1f)
            ) {
                val ingredientState = rememberLazyListState()
                val stepsState = rememberLazyListState()

                Box(
                    modifier = Modifier.fillMaxSize()
                ) {
                    Column(
                        modifier = Modifier.padding(top = 64.dp)
                    ) {
                        recipeHeader(recipe, true, slider)

                        if(recipe.linkedRecipes.isNotEmpty())
                        {
                            Column(modifier = Modifier.padding(all = 16.dp)) {
                                Text(text = "Recipes Referenced in this Recipe:")

                                Row {
                                    for (link in recipe.linkedRecipes) {
                                        Button(onClick = {
                                            navController.navigate(
                                                RecipeAppScreen.Details.name.plus(
                                                    "/${link.title}"
                                                )
                                            )
                                        }) {
                                            Text(text = link.title)
                                        }
                                    }
                                }
                            }
                        }

                        RecipeScaleSlider(
                            sliderPosition = slider,
                            steps = if(reduce) 2 else 5,
                            rangeEnd = if(reduce) 1f else 4f,
                            rangeStart = if(reduce) .25f else 1f,
                            reduce = reduce,
                            onClick = {
                                reduce = !reduce
                                slider = 1f
                            }
                        ) { slider = it}

                        Row {
                            Column(modifier = Modifier.weight(.5f)) {
                                Box(modifier = Modifier.padding(32.dp)) {
                                    Image(
                                        painter = painterResource(recipe.image),
                                        contentDescription = null,
                                        modifier = Modifier.aspectRatio(1f).align(Alignment.Center)
                                            .padding(16.dp)
                                            .size(400.dp).clip(CircleShape)
                                    )
                                }
                            }
                            Column(modifier = Modifier.weight(.5f)) {
                                ingredientsHeader(recipe)
                                LazyColumn(
                                    contentPadding = PaddingValues(64.dp),
                                    userScrollEnabled = true,
                                    verticalArrangement = Arrangement.Absolute.spacedBy(16.dp),
                                    modifier = Modifier.nestedScroll(nestedScrollConnection),
                                    state = ingredientState
                                ) {
                                    ingredients(recipe, slider)
                                }
                            }
                            Column(modifier = Modifier.weight(1f)) {
                                stepsHeader(recipe)
                                LazyColumn(
                                    contentPadding = PaddingValues(64.dp),
                                    userScrollEnabled = true,
                                    verticalArrangement = Arrangement.Absolute.spacedBy(16.dp),
                                    modifier = Modifier.nestedScroll(nestedScrollConnection),
                                    state = stepsState
                                ) {
                                    steps(recipe, slider)
                                }
                            }
                        }
                    }
                }
            }
        }

        Row {
            BackButton(goBack, true)
        }
    }
}

@Composable
fun BackButton(goBack: () -> Unit, isLarge: Boolean) {
    Box(
        modifier = Modifier.padding(
            start = if (isLarge) 32.dp else 12.dp,
            top =  if (isLarge) 16.dp else 8.dp,
            end = if (isLarge) 32.dp else 12.dp,
        ).clip(
            RoundedCornerShape(50)
        ).clickable {
            goBack()
        }.background(
            color = Color.Black, shape = RoundedCornerShape(50)
        )
    ) {
        Row(verticalAlignment = Alignment.CenterVertically) {
            Icon(
                imageVector = Icons.AutoMirrored.Default.ArrowBack,
                contentDescription = null,
                tint = Color.White,
                modifier = Modifier.size(20.dp)
            )
            if(isLarge) {
                Spacer(Modifier.padding(start = 8.dp))
                Text(
                    text = "Back to Recipes",
                    style = MaterialTheme.typography.headlineSmall,
                    color = Color.White
                )
            }
        }
    }
}


