package details

import AnimateInEffect
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyListScope
import androidx.compose.foundation.lazy.LazyRow
import androidx.compose.foundation.lazy.itemsIndexed
import androidx.compose.foundation.text.selection.SelectionContainer
import androidx.compose.material3.HorizontalDivider
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.font.FontStyle
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp
import model.Recipe


@Composable
fun recipeHeader(recipe: Recipe, isLarge: Boolean, slider: Float)
{
    if(isLarge) {
        SelectionContainer {
            Text(
                text = recipe.title,
                style = MaterialTheme.typography.headlineLarge,
                fontWeight = FontWeight.W700,
                modifier = Modifier.padding(top = 16.dp, start = 16.dp, end = 16.dp)
            )
        }
    }

    Text(
        text = recipe.description,
        style = if(isLarge) MaterialTheme.typography.headlineSmall else MaterialTheme.typography.bodyMedium,
        modifier = Modifier.padding(top = 16.dp, start = 16.dp, end = 16.dp)
    )

    val tags = recipe.tags.keys.toList()


    LazyRow (modifier = Modifier.padding(start = 16.dp, top = 16.dp)) {
        items(tags.size) { item ->
            Text(text = tags[item].plus( if(item == tags.size - 1) "" else ","),
                fontStyle = FontStyle.Italic,
                style = if(isLarge) MaterialTheme.typography.headlineSmall else MaterialTheme.typography.bodySmall,

            )
            Spacer(modifier = Modifier.width(3.dp))
        }
    }


    Row {
        Text(
            text = "Prep Time: ".plus(recipe.prepTime.toString()),
            style = if(isLarge) MaterialTheme.typography.bodyLarge else MaterialTheme.typography.bodyMedium,
            modifier = Modifier.padding(top = 16.dp, start = 16.dp, end = 16.dp)
        )
        Text(
            text = "Cook Time: ".plus(recipe.cookTime.toString()),
            style = if(isLarge) MaterialTheme.typography.bodyLarge else MaterialTheme.typography.bodyMedium,
            modifier = Modifier.padding(top = 16.dp, start = 16.dp, end = 16.dp)
        )
    }
    Row {
        Text(
            text = "Total Time: ".plus(recipe.cookTime.plus(recipe.prepTime).toString()),
            style = if(isLarge) MaterialTheme.typography.bodyLarge else MaterialTheme.typography.bodyMedium,
            modifier = Modifier.padding(top = 16.dp, start = 16.dp, end = 16.dp)
        )
        Text(
            text = "Serving Size: ".plus(scaleRecipeServings(recipe.servings, slider)),
            style = if(isLarge) MaterialTheme.typography.bodyLarge else MaterialTheme.typography.bodyMedium,
            modifier = Modifier.padding(top = 16.dp, start = 16.dp, end = 16.dp)
        )
    }
}

@Composable
fun ingredientsHeader(recipe: Recipe)
{
    AnimateInEffect(
        recipe = recipe,
        intervalStart = 0 / (recipe.instructions.size + recipe.ingredients.size + 2).toFloat(),
        content = {
            Text(
                text = "INGREDIENTS",
                style = MaterialTheme.typography.headlineMedium,
                fontWeight = FontWeight.W700,
                modifier = Modifier.padding(top = 16.dp, start = 16.dp, end = 16.dp)
            )
        })
}

@Composable
fun stepsHeader(recipe: Recipe)
{
    AnimateInEffect(
        recipe = recipe,
        intervalStart = (recipe.ingredients.size + 1) / (recipe.instructions.size + recipe.ingredients.size + 2).toFloat(),
        content = {
            Text(
                text = "STEPS",
                style = MaterialTheme.typography.headlineMedium,
                fontWeight = FontWeight.W700,
                modifier = Modifier.padding(top = 16.dp, start = 16.dp, end = 16.dp)
            )
        }
    )
}

internal fun LazyListScope.ingredients(
    recipe: Recipe,
    slider: Float
){
    itemsIndexed(recipe.ingredients) { index, value ->
        AnimateInEffect(
            intervalStart = (index + 1) / (recipe.ingredients.size + 1).toFloat(),
            recipe = recipe,
            content = {
                if (value.contains("HEADER-")) {
                    Column {
                        HorizontalDivider(
                            thickness = 2.dp,
                            color = if (recipe.bgColor != MaterialTheme.colorScheme.primaryContainer) MaterialTheme.colorScheme.secondaryContainer else MaterialTheme.colorScheme.background,
                            modifier = Modifier.padding(top = 16.dp)
                        )
                        Text(
                            text = value.removePrefix("HEADER-"),
                            style = MaterialTheme.typography.headlineSmall,
                            modifier = Modifier.padding(top = 16.dp, start = 16.dp, end = 16.dp)
                        )

                    }
                } else {
                    IngredientItem(recipe, scaleRecipeServings(value, slider))
                }
            }
        )
    }
}

internal fun LazyListScope.steps(
    recipe: Recipe,
    slider: Float
)
{
    itemsIndexed(recipe.instructions) { index, value ->
        AnimateInEffect(
            recipe = recipe,
            intervalStart = (index + 1) / (recipe.instructions.size + 1).toFloat(),
            content = {
                if (value.contains("HEADER-")) {
                    Column {
                        HorizontalDivider(
                            thickness = 2.dp,
                            color = if (recipe.bgColor != MaterialTheme.colorScheme.primaryContainer) MaterialTheme.colorScheme.secondaryContainer else MaterialTheme.colorScheme.background,
                            modifier = Modifier.padding(top = 16.dp)
                        )
                        Text(
                            text = value.removePrefix("HEADER-"),
                            style = MaterialTheme.typography.headlineSmall,
                            modifier = Modifier.padding(top = 16.dp, start = 16.dp, end = 16.dp)
                        )
                    }
                }else {
                InstructionItem(recipe, index, slider)
                }
            })
    }
}

fun scaleRecipeServingsInstructions(input: String, value: Float): String {
    val regex = Regex("\\[[+-]?([0-9]*[.])?[0-9]+")
    return regex.replace(input) { matchResult ->
        val number = matchResult.value.removePrefix("[").toFloat()
        "[".plus(removeTrailingZeros(number * value).toString())
    }
}

fun scaleRecipeServings(input: String, value: Float): String {
    val regex = Regex("[+-]?([0-9]*[.])?[0-9]+")
    return regex.replace(input) { matchResult ->
        val number = matchResult.value.toFloat()
        removeTrailingZeros(number * value)
    }
}

fun removeTrailingZeros(number: Float): String {
    return if(number.toInt().toFloat() == number)
        number.toInt().toString()
    else
        number.toString()
}